package sun.awt;

import java.util.Vector;
import java.util.Collections;
import java.awt.GraphicsConfiguration;
import java.util.WeakHashMap;
import sun.util.logging.PlatformLogger;
import java.awt.DefaultKeyboardFocusManager;
import java.awt.event.KeyEvent;
import java.awt.peer.MouseInfoPeer;
import java.awt.event.WindowEvent;
import sun.security.action.GetPropertyAction;
import sun.awt.im.SimpleInputMethodWindow;
import sun.awt.im.InputContext;
import sun.security.util.SecurityConstants;
import java.awt.image.DataBufferInt;
import java.awt.Graphics2D;
import java.util.Iterator;
import java.awt.image.BufferedImage;
import java.security.Permission;
import java.net.SocketPermission;
import java.io.FilePermission;
import sun.net.util.URLUtil;
import java.io.InputStream;
import java.io.IOException;
import java.io.File;
import java.awt.image.ImageObserver;
import sun.awt.image.MultiResolutionToolkitImage;
import sun.awt.image.ToolkitImage;
import java.awt.image.ImageProducer;
import sun.awt.image.ByteArrayImageSource;
import sun.awt.image.MultiResolutionImage;
import sun.awt.image.FileImageSource;
import sun.awt.image.URLImageSource;
import java.awt.Image;
import java.net.URL;
import java.security.AccessController;
import sun.security.action.GetBooleanAction;
import java.awt.peer.CanvasPeer;
import java.awt.Canvas;
import java.awt.peer.PanelPeer;
import java.awt.Panel;
import sun.font.FontDesignMetrics;
import java.awt.FontMetrics;
import java.awt.Font;
import java.awt.Dimension;
import java.lang.reflect.InvocationTargetException;
import java.awt.AWTEvent;
import java.awt.FocusTraversalPolicy;
import java.awt.KeyboardFocusManager;
import java.awt.Container;
import java.awt.MenuComponent;
import java.awt.Component;
import java.awt.GraphicsEnvironment;
import java.util.concurrent.TimeUnit;
import java.awt.peer.KeyboardFocusManagerPeer;
import java.awt.peer.RobotPeer;
import java.awt.GraphicsDevice;
import java.awt.Robot;
import java.awt.peer.FontPeer;
import java.awt.peer.SystemTrayPeer;
import java.awt.SystemTray;
import java.awt.AWTException;
import java.awt.peer.TrayIconPeer;
import java.awt.TrayIcon;
import java.awt.dnd.InvalidDnDOperationException;
import java.awt.dnd.peer.DragSourceContextPeer;
import java.awt.dnd.DragGestureEvent;
import java.awt.peer.CheckboxMenuItemPeer;
import java.awt.CheckboxMenuItem;
import java.awt.peer.MenuItemPeer;
import java.awt.MenuItem;
import java.awt.peer.PopupMenuPeer;
import java.awt.PopupMenu;
import java.awt.peer.MenuPeer;
import java.awt.Menu;
import java.awt.peer.MenuBarPeer;
import java.awt.MenuBar;
import java.awt.peer.FileDialogPeer;
import java.awt.FileDialog;
import java.awt.peer.TextAreaPeer;
import java.awt.TextArea;
import java.awt.peer.ScrollPanePeer;
import java.awt.ScrollPane;
import java.awt.peer.ScrollbarPeer;
import java.awt.Scrollbar;
import java.awt.peer.CheckboxPeer;
import java.awt.Checkbox;
import java.awt.peer.ListPeer;
import java.awt.List;
import java.awt.peer.LabelPeer;
import java.awt.Label;
import java.awt.peer.ChoicePeer;
import java.awt.Choice;
import java.awt.peer.TextFieldPeer;
import java.awt.TextField;
import java.awt.peer.ButtonPeer;
import java.awt.Button;
import java.awt.peer.DialogPeer;
import java.awt.peer.FramePeer;
import java.awt.Frame;
import java.awt.HeadlessException;
import java.awt.peer.WindowPeer;
import java.awt.Window;
import java.awt.EventQueue;
import java.awt.RenderingHints;
import java.awt.Dialog;
import java.util.Locale;
import sun.misc.SoftCache;
import java.util.Map;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;
import java.awt.Toolkit;

public abstract class SunToolkit extends Toolkit implements WindowClosingSupport, WindowClosingListener, ComponentFactory, InputMethodSupport, KeyboardFocusManagerPeerProvider
{
    public static final int GRAB_EVENT_MASK = Integer.MIN_VALUE;
    private static final String POST_EVENT_QUEUE_KEY = "PostEventQueue";
    protected static int numberOfButtons;
    public static final int MAX_BUTTONS_SUPPORTED = 20;
    private static final ReentrantLock AWT_LOCK;
    private static final Condition AWT_LOCK_COND;
    private static final Map<Object, AppContext> appContextMap;
    static final SoftCache fileImgCache;
    static final SoftCache urlImgCache;
    private static Locale startupLocale;
    private transient WindowClosingListener windowClosingListener;
    private static DefaultMouseInfoPeer mPeer;
    private static Dialog.ModalExclusionType DEFAULT_MODAL_EXCLUSION_TYPE;
    private SunToolkit.ModalityListenerList modalityListeners;
    public static final int DEFAULT_WAIT_TIME = 10000;
    private static final int MAX_ITERS = 20;
    private static final int MIN_ITERS = 0;
    private static final int MINIMAL_EDELAY = 0;
    private boolean eventDispatched;
    private boolean queueEmpty;
    private final Object waitLock;
    private static boolean touchKeyboardAutoShowIsEnabled;
    private static boolean checkedSystemAAFontSettings;
    private static boolean useSystemAAFontSettings;
    private static boolean lastExtraCondition;
    private static RenderingHints desktopFontHints;
    public static final String DESKTOPFONTHINTS = "awt.font.desktophints";
    private static Boolean sunAwtDisableMixing;
    private static final Object DEACTIVATION_TIMES_MAP_KEY;
    
    private static void initEQ(AppContext appContext) {
        String property = System.getProperty("AWT.EventQueueClass", "java.awt.EventQueue");
        EventQueue eventQueue;
        try {
            eventQueue = (EventQueue)Class.forName(property).newInstance();
        }
        catch (Exception obj) {
            obj.printStackTrace();
            System.err.println("Failed loading " + property + ": " + obj);
            eventQueue = new EventQueue();
        }
        appContext.put(AppContext.EVENT_QUEUE_KEY, eventQueue);
        appContext.put("PostEventQueue", new PostEventQueue(eventQueue));
    }
    
    public SunToolkit() {
        this.windowClosingListener = null;
        this.modalityListeners = new SunToolkit.ModalityListenerList();
        this.eventDispatched = false;
        this.queueEmpty = false;
        this.waitLock = "Wait Lock";
    }
    
    public boolean useBufferPerWindow() {
        return false;
    }
    
    @Override
    public abstract WindowPeer createWindow(Window p0) throws HeadlessException;
    
    @Override
    public abstract FramePeer createFrame(Frame p0) throws HeadlessException;
    
    public abstract FramePeer createLightweightFrame(LightweightFrame p0) throws HeadlessException;
    
    @Override
    public abstract DialogPeer createDialog(Dialog p0) throws HeadlessException;
    
    @Override
    public abstract ButtonPeer createButton(Button p0) throws HeadlessException;
    
    @Override
    public abstract TextFieldPeer createTextField(TextField p0) throws HeadlessException;
    
    @Override
    public abstract ChoicePeer createChoice(Choice p0) throws HeadlessException;
    
    @Override
    public abstract LabelPeer createLabel(Label p0) throws HeadlessException;
    
    @Override
    public abstract ListPeer createList(List p0) throws HeadlessException;
    
    @Override
    public abstract CheckboxPeer createCheckbox(Checkbox p0) throws HeadlessException;
    
    @Override
    public abstract ScrollbarPeer createScrollbar(Scrollbar p0) throws HeadlessException;
    
    @Override
    public abstract ScrollPanePeer createScrollPane(ScrollPane p0) throws HeadlessException;
    
    @Override
    public abstract TextAreaPeer createTextArea(TextArea p0) throws HeadlessException;
    
    @Override
    public abstract FileDialogPeer createFileDialog(FileDialog p0) throws HeadlessException;
    
    @Override
    public abstract MenuBarPeer createMenuBar(MenuBar p0) throws HeadlessException;
    
    @Override
    public abstract MenuPeer createMenu(Menu p0) throws HeadlessException;
    
    @Override
    public abstract PopupMenuPeer createPopupMenu(PopupMenu p0) throws HeadlessException;
    
    @Override
    public abstract MenuItemPeer createMenuItem(MenuItem p0) throws HeadlessException;
    
    @Override
    public abstract CheckboxMenuItemPeer createCheckboxMenuItem(CheckboxMenuItem p0) throws HeadlessException;
    
    @Override
    public abstract DragSourceContextPeer createDragSourceContextPeer(DragGestureEvent p0) throws InvalidDnDOperationException;
    
    public abstract TrayIconPeer createTrayIcon(TrayIcon p0) throws HeadlessException, AWTException;
    
    public abstract SystemTrayPeer createSystemTray(SystemTray p0);
    
    public abstract boolean isTraySupported();
    
    @Override
    public abstract FontPeer getFontPeer(String p0, int p1);
    
    @Override
    public abstract RobotPeer createRobot(Robot p0, GraphicsDevice p1) throws AWTException;
    
    @Override
    public abstract KeyboardFocusManagerPeer getKeyboardFocusManagerPeer() throws HeadlessException;
    
    public static final void awtLock() {
        SunToolkit.AWT_LOCK.lock();
    }
    
    public static final boolean awtTryLock() {
        return SunToolkit.AWT_LOCK.tryLock();
    }
    
    public static final void awtUnlock() {
        SunToolkit.AWT_LOCK.unlock();
    }
    
    public static final void awtLockWait() throws InterruptedException {
        SunToolkit.AWT_LOCK_COND.await();
    }
    
    public static final void awtLockWait(long n) throws InterruptedException {
        SunToolkit.AWT_LOCK_COND.await(n, TimeUnit.MILLISECONDS);
    }
    
    public static final void awtLockNotify() {
        SunToolkit.AWT_LOCK_COND.signal();
    }
    
    public static final void awtLockNotifyAll() {
        SunToolkit.AWT_LOCK_COND.signalAll();
    }
    
    public static final boolean isAWTLockHeldByCurrentThread() {
        return SunToolkit.AWT_LOCK.isHeldByCurrentThread();
    }
    
    public static AppContext createNewAppContext() {
        return createNewAppContext(Thread.currentThread().getThreadGroup());
    }
    
    static final AppContext createNewAppContext(ThreadGroup threadGroup) {
        AppContext appContext = new AppContext(threadGroup);
        initEQ(appContext);
        return appContext;
    }
    
    static void wakeupEventQueue(EventQueue eventQueue, boolean b) {
        AWTAccessor.getEventQueueAccessor().wakeup(eventQueue, b);
    }
    
    protected static Object targetToPeer(Object o) {
        if (o != null && !GraphicsEnvironment.isHeadless()) {
            return AWTAutoShutdown.getInstance().getPeer(o);
        }
        return null;
    }
    
    protected static void targetCreatedPeer(Object o, Object o2) {
        if (o != null && o2 != null && !GraphicsEnvironment.isHeadless()) {
            AWTAutoShutdown.getInstance().registerPeer(o, o2);
        }
    }
    
    protected static void targetDisposedPeer(Object o, Object o2) {
        if (o != null && o2 != null && !GraphicsEnvironment.isHeadless()) {
            AWTAutoShutdown.getInstance().unregisterPeer(o, o2);
        }
    }
    
    private static boolean setAppContext(Object o, AppContext appContext) {
        if ((o instanceof Component)) {
            AWTAccessor.getComponentAccessor().setAppContext((Component)o, appContext);
        }
        else if ((o instanceof MenuComponent)) {
            AWTAccessor.getMenuComponentAccessor().setAppContext((MenuComponent)o, appContext);
        }
        else {
            return false;
        }
        return true;
    }
    
    private static AppContext getAppContext(Object o) {
        if ((o instanceof Component)) {
            return AWTAccessor.getComponentAccessor().getAppContext((Component)o);
        }
        if ((o instanceof MenuComponent)) {
            return AWTAccessor.getMenuComponentAccessor().getAppContext((MenuComponent)o);
        }
        return null;
    }
    
    public static AppContext targetToAppContext(Object o) {
        if (o == null) {
            return null;
        }
        AppContext appContext = getAppContext(o);
        if (appContext == null) {
            appContext = (AppContext)SunToolkit.appContextMap.get(o);
            return appContext;
        }
        return appContext;
    }
    
    public static void setLWRequestStatus(Window window, boolean b) {
        AWTAccessor.getWindowAccessor().setLWRequestStatus(window, b);
    }
    
    public static void checkAndSetPolicy(Container container) {
        container.setFocusTraversalPolicy(KeyboardFocusManager.getCurrentKeyboardFocusManager().getDefaultFocusTraversalPolicy());
    }
    
    private static FocusTraversalPolicy createLayoutPolicy() {
        FocusTraversalPolicy focusTraversalPolicy = null;
        try {
            focusTraversalPolicy = (FocusTraversalPolicy)Class.forName("javax.swing.LayoutFocusTraversalPolicy").newInstance();
        }
        catch (ClassNotFoundException ex) {
            assert false;
        }
        catch (InstantiationException ex2) {
            assert false;
        }
        catch (IllegalAccessException ex3) {
            assert false;
        }
        return focusTraversalPolicy;
    }
    
    public static void insertTargetMapping(Object o, AppContext appContext) {
        if (!setAppContext(o, appContext)) {
            SunToolkit.appContextMap.put(o, appContext);
        }
    }
    
    public static void postEvent(AppContext obj, AWTEvent awtEvent) {
        if (awtEvent == null) {
            throw new NullPointerException();
        }
        AWTAccessor.SequencedEventAccessor sequencedEventAccessor = AWTAccessor.getSequencedEventAccessor();
        if (sequencedEventAccessor != null && sequencedEventAccessor.isSequencedEvent(awtEvent)) {
            AWTEvent nested = sequencedEventAccessor.getNested(awtEvent);
            if (nested.getID() == 208 && nested instanceof TimedWindowEvent) {
                TimedWindowEvent timedWindowEvent = (TimedWindowEvent)nested;
                ((SunToolkit)Toolkit.getDefaultToolkit()).setWindowDeactivationTime((Window)timedWindowEvent.getSource(), timedWindowEvent.getWhen());
            }
        }
        setSystemGenerated(awtEvent);
        AppContext targetToAppContext = targetToAppContext(awtEvent.getSource());
        if (targetToAppContext != null && !targetToAppContext.equals(obj)) {
            throw new RuntimeException("Event posted on wrong app context : " + awtEvent);
        }
        PostEventQueue postEventQueue = (PostEventQueue)obj.get("PostEventQueue");
        if (postEventQueue != null) {
            postEventQueue.postEvent(awtEvent);
        }
        return;
    }
    
    public static void postPriorityEvent(final AWTEvent awtEvent) {
        postEvent(targetToAppContext(awtEvent.getSource()), new PeerEvent(Toolkit.getDefaultToolkit(), new Runnable() {
            @Override
            public void run() {
                AWTAccessor.getAWTEventAccessor().setPosted(awtEvent);
                ((Component)awtEvent.getSource()).dispatchEvent(awtEvent);
            }
        }, 2L));
    }
    
    public static void flushPendingEvents() {
        flushPendingEvents(AppContext.getAppContext());
    }
    
    public static void flushPendingEvents(AppContext appContext) {
        PostEventQueue postEventQueue = (PostEventQueue)appContext.get("PostEventQueue");
        if (postEventQueue != null) {
            postEventQueue.flush();
        }
    }
    
    public static void executeOnEventHandlerThread(Object o, Runnable runnable) {
        executeOnEventHandlerThread(new PeerEvent(o, runnable, 1L));
    }
    
    public static void executeOnEventHandlerThread(final Object o, final Runnable runnable, final long n) {
        executeOnEventHandlerThread(new PeerEvent(o, runnable, 1L) {
            @Override
            public long getWhen() {
                return n;
            }
        });
    }
    
    public static void executeOnEventHandlerThread(PeerEvent peerEvent) {
        postEvent(targetToAppContext(peerEvent.getSource()), peerEvent);
    }
    
    public static void invokeLaterOnAppContext(AppContext appContext, Runnable runnable) {
        postEvent(appContext, new PeerEvent(Toolkit.getDefaultToolkit(), runnable, 1L));
    }
    
    public static void executeOnEDTAndWait(Object o, Runnable runnable) throws InterruptedException, InvocationTargetException {
        if (EventQueue.isDispatchThread()) {
            throw new Error("Cannot call executeOnEDTAndWait from any event dispatcher thread");
        }
        class AWTInvocationLock
        {
        }
        SunToolkit.1AWTInvocationLock awtInvocationLock = new AWTInvocationLock();
        PeerEvent peerEvent = new PeerEvent(o, runnable, awtInvocationLock, true, 1L);
        synchronized (awtInvocationLock) {
            executeOnEventHandlerThread(peerEvent);
            while (!peerEvent.isDispatched()) {
                awtInvocationLock.wait();
            }
        }
        Throwable throwable = peerEvent.getThrowable();
        if (throwable != null) {
            throw new InvocationTargetException(throwable);
        }
        return;
    }
    
    public static boolean isDispatchThreadForAppContext(Object o) {
        return AWTAccessor.getEventQueueAccessor().isDispatchThreadImpl((EventQueue)targetToAppContext(o).get(AppContext.EVENT_QUEUE_KEY));
    }
    
    @Override
    public Dimension getScreenSize() {
        return new Dimension(this.getScreenWidth(), this.getScreenHeight());
    }
    
    protected abstract int getScreenWidth();
    
    protected abstract int getScreenHeight();
    
    @Override
    public FontMetrics getFontMetrics(Font font) {
        return FontDesignMetrics.getMetrics(font);
    }
    
    @Override
    public String[] getFontList() {
        return new String[] { "Dialog", "SansSerif", "Serif", "Monospaced", "DialogInput" };
    }
    
    @Override
    public PanelPeer createPanel(Panel panel) {
        return (PanelPeer)this.createComponent(panel);
    }
    
    @Override
    public CanvasPeer createCanvas(Canvas canvas) {
        return (CanvasPeer)this.createComponent(canvas);
    }
    
    public void disableBackgroundErase(Canvas canvas) {
        this.disableBackgroundEraseImpl(canvas);
    }
    
    public void disableBackgroundErase(Component component) {
        this.disableBackgroundEraseImpl(component);
    }
    
    private void disableBackgroundEraseImpl(Component component) {
        AWTAccessor.getComponentAccessor().setBackgroundEraseDisabled(component, true);
    }
    
    public static boolean getSunAwtNoerasebackground() {
        return (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.awt.noerasebackground"));
    }
    
    public static boolean getSunAwtErasebackgroundonresize() {
        return (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.awt.erasebackgroundonresize"));
    }
    
    static Image getImageFromHash(Toolkit toolkit, URL u) {
        checkPermissions(u);
        synchronized (SunToolkit.urlImgCache) {
            String string = u.toString();
            Image image = (Image)SunToolkit.urlImgCache.get(string);
            if (image == null) {
                try {
                    image = toolkit.createImage(new URLImageSource(u));
                    SunToolkit.urlImgCache.put(string, image);
                }
                catch (Exception ex) {}
            }
            return image;
        }
    }
    
    static Image getImageFromHash(Toolkit toolkit, String filename) {
        checkPermissions(filename);
        synchronized (SunToolkit.fileImgCache) {
            Image image = (Image)SunToolkit.fileImgCache.get(filename);
            if (image == null) {
                try {
                    image = toolkit.createImage(new FileImageSource(filename));
                    SunToolkit.fileImgCache.put(filename, image);
                }
                catch (Exception ex) {}
            }
            return image;
        }
    }
    
    @Override
    public Image getImage(String s) {
        return getImageFromHash(this, s);
    }
    
    @Override
    public Image getImage(URL url) {
        return getImageFromHash(this, url);
    }
    
    protected Image getImageWithResolutionVariant(String s, String s2) {
        synchronized (SunToolkit.fileImgCache) {
            Image imageFromHash = getImageFromHash(this, s);
            if ((imageFromHash instanceof MultiResolutionImage)) {
                return imageFromHash;
            }
            Image imageWithResolutionVariant = createImageWithResolutionVariant(imageFromHash, getImageFromHash(this, s2));
            SunToolkit.fileImgCache.put(s, imageWithResolutionVariant);
            return imageWithResolutionVariant;
        }
    }
    
    protected Image getImageWithResolutionVariant(URL url, URL url2) {
        synchronized (SunToolkit.urlImgCache) {
            Image imageFromHash = getImageFromHash(this, url);
            if ((imageFromHash instanceof MultiResolutionImage)) {
                return imageFromHash;
            }
            Image imageWithResolutionVariant = createImageWithResolutionVariant(imageFromHash, getImageFromHash(this, url2));
            SunToolkit.urlImgCache.put(url.toString(), imageWithResolutionVariant);
            return imageWithResolutionVariant;
        }
    }
    
    @Override
    public Image createImage(String filename) {
        checkPermissions(filename);
        return this.createImage(new FileImageSource(filename));
    }
    
    @Override
    public Image createImage(URL u) {
        checkPermissions(u);
        return this.createImage(new URLImageSource(u));
    }
    
    @Override
    public Image createImage(byte[] data, int offset, int length) {
        return this.createImage(new ByteArrayImageSource(data, offset, length));
    }
    
    @Override
    public Image createImage(ImageProducer is) {
        return new ToolkitImage(is);
    }
    
    public static Image createImageWithResolutionVariant(Image lowResolutionImage, Image resolutionVariant) {
        return new MultiResolutionToolkitImage(lowResolutionImage, resolutionVariant);
    }
    
    @Override
    public int checkImage(Image image, int n, int n2, ImageObserver imageObserver) {
        if (!(image instanceof ToolkitImage)) {
            return 32;
        }
        ToolkitImage toolkitImage = (ToolkitImage)image;
        int check;
        if (n == 0 || n2 == 0) {
            check = 32;
        }
        else {
            check = toolkitImage.getImageRep().check(imageObserver);
        }
        return (toolkitImage.check(imageObserver) | check) & this.checkResolutionVariant(image, n, n2, imageObserver);
    }
    
    @Override
    public boolean prepareImage(Image image, int n, int n2, ImageObserver iw) {
        if (n == 0 || n2 == 0) {
            return true;
        }
        if (!(image instanceof ToolkitImage)) {
            return true;
        }
        ToolkitImage toolkitImage = (ToolkitImage)image;
        if (toolkitImage.hasError()) {
            if (iw != null) {
                iw.imageUpdate(image, 192, -1, -1, -1, -1);
            }
            return false;
        }
        else {
            return toolkitImage.getImageRep().prepare(iw) & this.prepareResolutionVariant(image, n, n2, iw);
        }
    }
    
    private int checkResolutionVariant(Image image, int imgWidth, int imgHeight, ImageObserver observer) {
        ToolkitImage resolutionVariant = getResolutionVariant(image);
        int rvSize = getRVSize(imgWidth);
        int rvSize2 = getRVSize(imgHeight);
        return (resolutionVariant == null || resolutionVariant.hasError()) ? 65535 : this.checkImage(resolutionVariant, rvSize, rvSize2, MultiResolutionToolkitImage.getResolutionVariantObserver(image, observer, imgWidth, imgHeight, rvSize, rvSize2, true));
    }
    
    private boolean prepareResolutionVariant(Image image, int imgWidth, int imgHeight, ImageObserver observer) {
        ToolkitImage resolutionVariant = getResolutionVariant(image);
        int rvSize = getRVSize(imgWidth);
        int rvSize2 = getRVSize(imgHeight);
        return resolutionVariant == null || resolutionVariant.hasError() || this.prepareImage(resolutionVariant, rvSize, rvSize2, MultiResolutionToolkitImage.getResolutionVariantObserver(image, observer, imgWidth, imgHeight, rvSize, rvSize2, true));
    }
    
    private static int getRVSize(int n) {
        return (n == -1) ? -1 : (2 * n);
    }
    
    private static ToolkitImage getResolutionVariant(Image image) {
        if ((image instanceof MultiResolutionToolkitImage)) {
            Image resolutionVariant = ((MultiResolutionToolkitImage)image).getResolutionVariant();
            if ((resolutionVariant instanceof ToolkitImage)) {
                return (ToolkitImage)resolutionVariant;
            }
        }
        return null;
    }
    
    protected static boolean imageCached(String s) {
        return SunToolkit.fileImgCache.containsKey(s);
    }
    
    protected static boolean imageCached(URL url) {
        return SunToolkit.urlImgCache.containsKey(url.toString());
    }
    
    protected static boolean imageExists(String pathname) {
        if (pathname != null) {
            checkPermissions(pathname);
            return new File(pathname).exists();
        }
        return false;
    }
    
    protected static boolean imageExists(URL url) {
        if (url != null) {
            checkPermissions(url);
            try (final InputStream openStream = url.openStream()) {
                return true;
            }
            catch (IOException ex) {
                return false;
            }
        }
        return false;
    }
    
    private static void checkPermissions(String file) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkRead(file);
        }
    }
    
    private static void checkPermissions(URL url) {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            try {
                Permission connectPermission = URLUtil.getConnectPermission(url);
                if (connectPermission != null) {
                    try {
                        securityManager.checkPermission(connectPermission);
                    }
                    catch (SecurityException ex) {
                        if (connectPermission instanceof FilePermission && connectPermission.getActions().indexOf("read") != -1) {
                            securityManager.checkRead(connectPermission.getName());
                        }
                        else if (connectPermission instanceof SocketPermission && connectPermission.getActions().indexOf("connect") != -1) {
                            securityManager.checkConnect(url.getHost(), url.getPort());
                        }
                        else {
                            throw ex;
                        }
                    }
                }
            }
            catch (IOException ex2) {
                securityManager.checkConnect(url.getHost(), url.getPort());
            }
        }
    }
    
    public static BufferedImage getScaledIconImage(java.util.List<Image> list, int width, int height) {
        if (width == 0 || height == 0) {
            return null;
        }
        Image image = null;
        int n = 0;
        int n2 = 0;
        double n3 = 3.0;
        for (Image image2 : list) {
            if (image2 == null) {
                continue;
            }
            if ((image2 instanceof ToolkitImage)) {
                ((ToolkitImage)image2).getImageRep().reconstruct(32);
            }
            int width2;
            int height2;
            try {
                width2 = image2.getWidth(null);
                height2 = image2.getHeight(null);
            }
            catch (Exception ex) {
                continue;
            }
            if (width2 > 0 && height2 > 0) {
                double min = Math.min((double)width / (double)width2, (double)height / (double)height2);
                int n4;
                int n5;
                double n6;
                if (min >= 2.0) {
                    double floor = Math.floor(min);
                    n4 = width2 * (int)floor;
                    n5 = height2 * (int)floor;
                    n6 = 1.0 - 0.5 / floor;
                }
                else if (min >= 1.0) {
                    double floor = 1.0;
                    n4 = width2;
                    n5 = height2;
                    n6 = 0.0;
                }
                else if (min >= 0.75) {
                    double floor = 0.75;
                    n4 = width2 * 3 / 4;
                    n5 = height2 * 3 / 4;
                    n6 = 0.3;
                }
                else if (min >= 0.6666) {
                    double floor = 0.6666;
                    n4 = width2 * 2 / 3;
                    n5 = height2 * 2 / 3;
                    n6 = 0.33;
                }
                else {
                    double ceil = Math.ceil(1.0 / min);
                    double floor = 1.0 / ceil;
                    n4 = (int)Math.round((double)width2 / ceil);
                    n5 = (int)Math.round((double)height2 / ceil);
                    n6 = 1.0 - 1.0 / ceil;
                }
                double n7 = ((double)width - (double)n4) / (double)width + ((double)height - (double)n5) / (double)height + n6;
                if (n7 < n3) {
                    n3 = n7;
                    image = image2;
                    n = n4;
                    n2 = n5;
                }
                if (n7 == 0.0) {
                    break;
                }
                continue;
            }
            else {
                continue;
            }
        }
        if (image == null) {
            return null;
        }
        BufferedImage o = new BufferedImage(width, height, 2);
        Graphics2D graphics = o.createGraphics();
        graphics.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
        try {
            graphics.drawImage(image, (width - n) / 2, (height - n2) / 2, n, n2, null);
        }
        finally {
            graphics.dispose();
        }
        return o;
    }
    
    public static DataBufferInt getScaledIconData(java.util.List<Image> list, int n, int n2) {
        BufferedImage scaledIconImage = getScaledIconImage(list, n, n2);
        if (scaledIconImage == null) {
            return null;
        }
        return (DataBufferInt)scaledIconImage.getRaster().getDataBuffer();
    }
    
    @Override
    protected EventQueue getSystemEventQueueImpl() {
        return getSystemEventQueueImplPP();
    }
    
    static EventQueue getSystemEventQueueImplPP() {
        return getSystemEventQueueImplPP(AppContext.getAppContext());
    }
    
    public static EventQueue getSystemEventQueueImplPP(AppContext appContext) {
        return (EventQueue)appContext.get(AppContext.EVENT_QUEUE_KEY);
    }
    
    public static Container getNativeContainer(Component c) {
        return Toolkit.getNativeContainer(c);
    }
    
    public static Component getHeavyweightComponent(Component parent) {
        while (parent != null && AWTAccessor.getComponentAccessor().isLightweight(parent)) {
            parent = AWTAccessor.getComponentAccessor().getParent(parent);
        }
        return parent;
    }
    
    public int getFocusAcceleratorKeyMask() {
        return 8;
    }
    
    public boolean isPrintableCharacterModifiersMask(int n) {
        return (n & 0x8) == (n & 0x2);
    }
    
    public boolean canPopupOverlapTaskBar() {
        boolean n = true;
        try {
            SecurityManager securityManager = System.getSecurityManager();
            if (securityManager != null) {
                securityManager.checkPermission(SecurityConstants.AWT.SET_WINDOW_ALWAYS_ON_TOP_PERMISSION);
            }
        }
        catch (SecurityException ex) {
            n = false;
        }
        return n;
    }
    
    @Override
    public Window createInputMethodWindow(String title, InputContext context) {
        return new SimpleInputMethodWindow(title, context);
    }
    
    @Override
    public boolean enableInputMethodsForTextComponent() {
        return false;
    }
    
    public static Locale getStartupLocale() {
        if (SunToolkit.startupLocale == null) {
            String language = (String)AccessController.doPrivileged(new GetPropertyAction("user.language", "en"));
            String s = (String)AccessController.doPrivileged(new GetPropertyAction("user.region"));
            String substring;
            String substring2;
            if (s != null) {
                int index = s.indexOf(95);
                if (index >= 0) {
                    substring = s.substring(0, index);
                    substring2 = s.substring(index + 1);
                }
                else {
                    substring = s;
                    substring2 = "";
                }
            }
            else {
                substring = (String)AccessController.doPrivileged(new GetPropertyAction("user.country", ""));
                substring2 = (String)AccessController.doPrivileged(new GetPropertyAction("user.variant", ""));
            }
            SunToolkit.startupLocale = new Locale(language, substring, substring2);
        }
        return SunToolkit.startupLocale;
    }
    
    @Override
    public Locale getDefaultKeyboardLocale() {
        return getStartupLocale();
    }
    
    @Override
    public WindowClosingListener getWindowClosingListener() {
        return this.windowClosingListener;
    }
    
    @Override
    public void setWindowClosingListener(WindowClosingListener windowClosingListener) {
        this.windowClosingListener = windowClosingListener;
    }
    
    @Override
    public RuntimeException windowClosingNotify(WindowEvent windowEvent) {
        if (this.windowClosingListener != null) {
            return this.windowClosingListener.windowClosingNotify(windowEvent);
        }
        return null;
    }
    
    @Override
    public RuntimeException windowClosingDelivered(WindowEvent windowEvent) {
        if (this.windowClosingListener != null) {
            return this.windowClosingListener.windowClosingDelivered(windowEvent);
        }
        return null;
    }
    
    protected synchronized MouseInfoPeer getMouseInfoPeer() {
        if (SunToolkit.mPeer == null) {
            SunToolkit.mPeer = new DefaultMouseInfoPeer();
        }
        return SunToolkit.mPeer;
    }
    
    public static boolean needsXEmbed() {
        if ("true".equals((String)AccessController.doPrivileged(new GetPropertyAction("sun.awt.noxembed", "false")))) {
            return false;
        }
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        return defaultToolkit instanceof SunToolkit && ((SunToolkit)defaultToolkit).needsXEmbedImpl();
    }
    
    protected boolean needsXEmbedImpl() {
        return false;
    }
    
    protected final boolean isXEmbedServerRequested() {
        return (boolean)AccessController.doPrivileged(new GetBooleanAction("sun.awt.xembedserver"));
    }
    
    public static boolean isModalExcludedSupported() {
        return Toolkit.getDefaultToolkit().isModalExclusionTypeSupported(SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE);
    }
    
    protected boolean isModalExcludedSupportedImpl() {
        return false;
    }
    
    public static void setModalExcluded(Window window) {
        if (SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE == null) {
            SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE = Dialog.ModalExclusionType.APPLICATION_EXCLUDE;
        }
        window.setModalExclusionType(SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE);
    }
    
    public static boolean isModalExcluded(Window window) {
        if (SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE == null) {
            SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE = Dialog.ModalExclusionType.APPLICATION_EXCLUDE;
        }
        return window.getModalExclusionType().compareTo(SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE) >= 0;
    }
    
    @Override
    public boolean isModalityTypeSupported(Dialog.ModalityType modalityType) {
        return modalityType == Dialog.ModalityType.MODELESS || modalityType == Dialog.ModalityType.APPLICATION_MODAL;
    }
    
    @Override
    public boolean isModalExclusionTypeSupported(Dialog.ModalExclusionType modalExclusionType) {
        return modalExclusionType == Dialog.ModalExclusionType.NO_EXCLUDE;
    }
    
    public void addModalityListener(ModalityListener modalityListener) {
        this.modalityListeners.add(modalityListener);
    }
    
    public void removeModalityListener(ModalityListener modalityListener) {
        this.modalityListeners.remove(modalityListener);
    }
    
    public void notifyModalityPushed(Dialog dialog) {
        this.notifyModalityChange(1300, dialog);
    }
    
    public void notifyModalityPopped(Dialog dialog) {
        this.notifyModalityChange(1301, dialog);
    }
    
    final void notifyModalityChange(int n, Dialog dialog) {
        new ModalityEvent(dialog, this.modalityListeners, n).dispatch();
    }
    
    public static boolean isLightweightOrUnknown(Component component) {
        return component.isLightweight() || !(getDefaultToolkit() instanceof SunToolkit) || (!(component instanceof Button) && !(component instanceof Canvas) && !(component instanceof Checkbox) && !(component instanceof Choice) && !(component instanceof Label) && !(component instanceof List) && !(component instanceof Panel) && !(component instanceof Scrollbar) && !(component instanceof ScrollPane) && !(component instanceof TextArea) && !(component instanceof TextField) && !(component instanceof Window));
    }
    
    public void realSync() throws SunToolkit.OperationTimedOut, SunToolkit.InfiniteLoop {
        this.realSync(10000L);
    }
    
    public void realSync(long n) throws SunToolkit.OperationTimedOut, SunToolkit.InfiniteLoop {
        if (EventQueue.isDispatchThread()) {
            throw new SunToolkit.IllegalThreadException("The SunToolkit.realSync() method cannot be used on the event dispatch thread (EDT).");
        }
        int n2 = 0;
        do {
            this.sync();
            int i;
            for (i = 0; i < 0; ++i) {
                this.syncNativeQueue(n);
            }
            while (this.syncNativeQueue(n) && i < 20) {
                ++i;
            }
            if (i >= 20) {
                throw new SunToolkit.InfiniteLoop();
            }
            int j;
            for (j = 0; j < 0; ++j) {
                this.waitForIdle(n);
            }
            while (this.waitForIdle(n) && j < 20) {
                ++j;
            }
            if (j >= 20) {
                throw new SunToolkit.InfiniteLoop();
            }
            ++n2;
        } while ((this.syncNativeQueue(n) || this.waitForIdle(n)) && n2 < 20);
        return;
    }
    
    protected abstract boolean syncNativeQueue(long p0);
    
    private boolean isEQEmpty() {
        return AWTAccessor.getEventQueueAccessor().noEvents(this.getSystemEventQueueImpl());
    }
    
    protected final boolean waitForIdle(final long n) {
        flushPendingEvents();
        int eqEmpty = this.isEQEmpty();
        this.queueEmpty = false;
        this.eventDispatched = false;
        synchronized (this.waitLock) {
            postEvent(AppContext.getAppContext(), new PeerEvent(this.getSystemEventQueueImpl(), null, 4L) {
                @Override
                public void dispatch() {
                    int i;
                    for (i = 0; i < 0; ++i) {
                        SunToolkit.this.syncNativeQueue(n);
                    }
                    while (SunToolkit.this.syncNativeQueue(n) && i < 20) {
                        ++i;
                    }
                    SunToolkit.flushPendingEvents();
                    synchronized (SunToolkit.this.waitLock) {
                        SunToolkit.this.queueEmpty = SunToolkit.access$200(SunToolkit.this);
                        SunToolkit.this.eventDispatched = true;
                        SunToolkit.this.waitLock.notifyAll();
                    }
                }
            });
            try {
                while (!this.eventDispatched) {
                    this.waitLock.wait();
                }
            }
            catch (InterruptedException ex) {
                return false;
            }
        }
        try {
            Thread.sleep(0L);
        }
        catch (InterruptedException ex2) {
            throw new RuntimeException("Interrupted");
        }
        flushPendingEvents();
        synchronized (this.waitLock) {
            return !this.queueEmpty || !this.isEQEmpty() || eqEmpty == 0;
        }
    }
    
    public abstract void grab(Window p0);
    
    public abstract void ungrab(Window p0);
    
    public void showOrHideTouchKeyboard(Component component, AWTEvent awtEvent) {
    }
    
    public static boolean isTouchKeyboardAutoShowEnabled() {
        return SunToolkit.touchKeyboardAutoShowIsEnabled;
    }
    
    public static native void closeSplashScreen();
    
    private void fireDesktopFontPropertyChanges() {
        this.setDesktopProperty("awt.font.desktophints", getDesktopFontHints());
    }
    
    public static void setAAFontSettingsCondition(boolean lastExtraCondition) {
        if (lastExtraCondition != SunToolkit.lastExtraCondition) {
            SunToolkit.lastExtraCondition = lastExtraCondition;
            if (SunToolkit.checkedSystemAAFontSettings) {
                SunToolkit.checkedSystemAAFontSettings = false;
                Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
                if ((defaultToolkit instanceof SunToolkit)) {
                    ((SunToolkit)defaultToolkit).fireDesktopFontPropertyChanges();
                }
            }
        }
    }
    
    private static RenderingHints getDesktopAAHintsByName(String lowerCase) {
        Object value = null;
        lowerCase = lowerCase.toLowerCase(Locale.ENGLISH);
        if (lowerCase.equals("on")) {
            value = RenderingHints.VALUE_TEXT_ANTIALIAS_ON;
        }
        else if (lowerCase.equals("gasp")) {
            value = RenderingHints.VALUE_TEXT_ANTIALIAS_GASP;
        }
        else if (lowerCase.equals("lcd") || lowerCase.equals("lcd_hrgb")) {
            value = RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB;
        }
        else if (lowerCase.equals("lcd_hbgr")) {
            value = RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HBGR;
        }
        else if (lowerCase.equals("lcd_vrgb")) {
            value = RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_VRGB;
        }
        else if (lowerCase.equals("lcd_vbgr")) {
            value = RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_VBGR;
        }
        if (value != null) {
            RenderingHints o = new RenderingHints(null);
            o.put(RenderingHints.KEY_TEXT_ANTIALIASING, value);
            return o;
        }
        return null;
    }
    
    private static boolean useSystemAAFontSettings() {
        if (!SunToolkit.checkedSystemAAFontSettings) {
            SunToolkit.useSystemAAFontSettings = true;
            String s = null;
            if ((Toolkit.getDefaultToolkit() instanceof SunToolkit)) {
                s = (String)AccessController.doPrivileged(new GetPropertyAction("awt.useSystemAAFontSettings"));
            }
            if (s != null && !(SunToolkit.useSystemAAFontSettings = (boolean)Boolean.valueOf(s))) {
                SunToolkit.desktopFontHints = getDesktopAAHintsByName(s);
            }
            if (SunToolkit.useSystemAAFontSettings) {
                SunToolkit.useSystemAAFontSettings = SunToolkit.lastExtraCondition;
            }
            SunToolkit.checkedSystemAAFontSettings = true;
        }
        return SunToolkit.useSystemAAFontSettings;
    }
    
    protected RenderingHints getDesktopAAHints() {
        return null;
    }
    
    public static RenderingHints getDesktopFontHints() {
        if (useSystemAAFontSettings()) {
            Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
            if ((defaultToolkit instanceof SunToolkit)) {
                return (RenderingHints)((SunToolkit)defaultToolkit).getDesktopAAHints();
            }
            return null;
        }
        else {
            if (SunToolkit.desktopFontHints != null) {
                return (RenderingHints)(RenderingHints)SunToolkit.desktopFontHints.clone();
            }
            return null;
        }
    }
    
    public abstract boolean isDesktopSupported();
    
    public static synchronized void consumeNextKeyTyped(KeyEvent keyEvent) {
        try {
            AWTAccessor.getDefaultKeyboardFocusManagerAccessor().consumeNextKeyTyped((DefaultKeyboardFocusManager)KeyboardFocusManager.getCurrentKeyboardFocusManager(), keyEvent);
        }
        catch (ClassCastException ex) {
            ex.printStackTrace();
        }
    }
    
    protected static void dumpPeers(PlatformLogger platformLogger) {
        AWTAutoShutdown.getInstance().dumpPeers(platformLogger);
    }
    
    public static Window getContainingWindow(Component parent) {
        while (parent != null && !(parent instanceof Window)) {
            parent = parent.getParent();
        }
        return (Window)parent;
    }
    
    public static synchronized boolean getSunAwtDisableMixing() {
        if (SunToolkit.sunAwtDisableMixing == null) {
            SunToolkit.sunAwtDisableMixing = (Boolean)AccessController.doPrivileged(new GetBooleanAction("sun.awt.disableMixing"));
        }
        return (boolean)SunToolkit.sunAwtDisableMixing;
    }
    
    public boolean isNativeGTKAvailable() {
        return false;
    }
    
    public synchronized void setWindowDeactivationTime(Window key, long l) {
        AppContext appContext = getAppContext(key);
        WeakHashMap<Window, Long> weakHashMap = (WeakHashMap)appContext.get(SunToolkit.DEACTIVATION_TIMES_MAP_KEY);
        if (weakHashMap == null) {
            weakHashMap = new WeakHashMap();
            appContext.put(SunToolkit.DEACTIVATION_TIMES_MAP_KEY, weakHashMap);
        }
        weakHashMap.put(key, Long.valueOf(l));
    }
    
    public synchronized long getWindowDeactivationTime(Window key) {
        WeakHashMap<Object, Object> weakHashMap = (WeakHashMap)getAppContext(key).get(SunToolkit.DEACTIVATION_TIMES_MAP_KEY);
        if (weakHashMap == null) {
            return -1L;
        }
        Long n = (Long)weakHashMap.get(key);
        return (n == null) ? -1L : ((long)n);
    }
    
    public boolean isWindowOpacitySupported() {
        return false;
    }
    
    public boolean isWindowShapingSupported() {
        return false;
    }
    
    public boolean isWindowTranslucencySupported() {
        return false;
    }
    
    public boolean isTranslucencyCapable(GraphicsConfiguration graphicsConfiguration) {
        return false;
    }
    
    public boolean isSwingBackbufferTranslucencySupported() {
        return false;
    }
    
    public static boolean isContainingTopLevelOpaque(Component component) {
        Window containingWindow = getContainingWindow(component);
        return containingWindow != null && containingWindow.isOpaque();
    }
    
    public static boolean isContainingTopLevelTranslucent(Component component) {
        Window containingWindow = getContainingWindow(component);
        return containingWindow != null && containingWindow.getOpacity() < 1.0f;
    }
    
    public boolean needUpdateWindow() {
        return false;
    }
    
    public int getNumberOfButtons() {
        return 3;
    }
    
    public static boolean isInstanceOf(Object o, String s) {
        return o != null && s != null && isInstanceOf(o.getClass(), s);
    }
    
    private static boolean isInstanceOf(Class<?> clazz, String s) {
        if (clazz == null) {
            return false;
        }
        if (clazz.getName().equals(s)) {
            return true;
        }
        Class[] interfaces = clazz.getInterfaces();
        int length = interfaces.length;
        int i = 0;
        while (i < length) {
            if (interfaces[i].getName().equals(s)) {
                return true;
            }
            ++i;
        }
        return isInstanceOf(clazz.getSuperclass(), s);
    }
    
    protected static LightweightFrame getLightweightFrame(Component parent) {
        while (parent != null) {
            if ((parent instanceof LightweightFrame)) {
                return (LightweightFrame)parent;
            }
            if ((parent instanceof Window)) {
                return null;
            }
            parent = parent.getParent();
        }
        return null;
    }
    
    public static void setSystemGenerated(AWTEvent systemGenerated) {
        AWTAccessor.getAWTEventAccessor().setSystemGenerated(systemGenerated);
    }
    
    public static boolean isSystemGenerated(AWTEvent awtEvent) {
        return AWTAccessor.getAWTEventAccessor().isSystemGenerated(awtEvent);
    }
    
    static boolean access$200(SunToolkit sunToolkit) {
        return sunToolkit.isEQEmpty();
    }
    
    static {
        if ((boolean)AccessController.doPrivileged(new GetBooleanAction("sun.awt.nativedebug"))) {
            DebugSettings.init();
        }
        SunToolkit.touchKeyboardAutoShowIsEnabled = (boolean)Boolean.valueOf((String)AccessController.doPrivileged(new GetPropertyAction("awt.touchKeyboardAutoShowIsEnabled", "true")));
        SunToolkit.numberOfButtons = 0;
        AWT_LOCK = new ReentrantLock();
        AWT_LOCK_COND = SunToolkit.AWT_LOCK.newCondition();
        appContextMap = Collections.synchronizedMap(new WeakHashMap());
        fileImgCache = new SoftCache();
        urlImgCache = new SoftCache();
        SunToolkit.startupLocale = null;
        SunToolkit.mPeer = null;
        SunToolkit.DEFAULT_MODAL_EXCLUSION_TYPE = null;
        SunToolkit.lastExtraCondition = true;
        SunToolkit.sunAwtDisableMixing = null;
        DEACTIVATION_TIMES_MAP_KEY = new Object();
    }
    
    static class ModalityListenerList implements ModalityListener
    {
        Vector<ModalityListener> listeners;
        
        ModalityListenerList() {
            this.listeners = new Vector();
        }
        
        void add(ModalityListener obj) {
            this.listeners.addElement(obj);
        }
        
        void remove(ModalityListener obj) {
            this.listeners.removeElement(obj);
        }
        
        @Override
        public void modalityPushed(ModalityEvent modalityEvent) {
            Iterator<ModalityListener> iterator = this.listeners.iterator();
            while (iterator.hasNext()) {
                ((ModalityListener)iterator.next()).modalityPushed(modalityEvent);
            }
        }
        
        @Override
        public void modalityPopped(ModalityEvent modalityEvent) {
            Iterator<ModalityListener> iterator = this.listeners.iterator();
            while (iterator.hasNext()) {
                ((ModalityListener)iterator.next()).modalityPopped(modalityEvent);
            }
        }
    }
    
    public static class OperationTimedOut extends RuntimeException
    {
        public OperationTimedOut(String message) {
            super(message);
        }
        
        public OperationTimedOut() {
        }
    }
    
    public static class InfiniteLoop extends RuntimeException
    {
    }
    
    public static class IllegalThreadException extends RuntimeException
    {
        public IllegalThreadException(String message) {
            super(message);
        }
        
        public IllegalThreadException() {
        }
    }
}
